DAT programName         byte "WearableKeypad130430b", 0

CON
{{
  ******* Public Notes *******

  Written By: Duane Degn
  Initial Release: May 2013
  MIT License (Use anyway you want but give me credit.)
  If you like it you can send me some money.

  
}}
{
  ******* Private Notes *******
  WearableKeypad130424e
  130424e Make all timer intervals multiples of a PWM period.
  30a The LEDs don't blink when switched to blink mode.
  30a Remove demo portions of object.
  30b Change keypad LED timing to milliseconds instead of PWM
  cycles.
}
CON

  PSEUDO_MULTIPLIER = 100
  MAX_BRIGHTNESS = $FF
  MIN_BRIGHTNESS = 0

  KEYPAD_LED_RESOLUTION_BITS = 5
  KEYPAD_LED_RESOLUTION = 1 << KEYPAD_LED_RESOLUTION_BITS
  KEYPAD_LED_RES_MAX_BRIGHTNESS = KEYPAD_LED_RESOLUTION - 1
  
  ' button enumeration
  #0, NO_BUTTON, CENTER_BUTTON, UP_BUTTON, DOWN_BUTTON, LEFT_BUTTON, RIGHT_BUTTON

  ' LED modes
  #0, OFF_LED, PULSE_LED, BLINK_LED, SOLID_LED

  ' offsets from brightnessAddress (all longs)
  #0, BRIGHTNESS_BLINK_OFFSET, BRIGHTNESS_PULSE_OFFSET, MODE_OFFSET, {RATE_OFFSET,} {
    } BLINK_NUMBER_OFFSET, LED_TIMER_PULSE_ON_OFFSET, LED_TIMER_PULSE_OFF_OFFSET, {
    } LED_TIMER_BLINK_ON_OFFSET, LED_TIMER_BLINK_OFF_OFFSET, {
    } LED_TIMER_BLINK_PAUSE_OFFSET
  
  ' phase enumeration
  #0, INCREASE_PHASE, DECREASE_PHASE, PAUSE_PHASE

  ' pwmFlag enumeration
  #0, OFF_PWM, ON_PWM
CON

  DEFAULT_BLINK_NUMBER = 3
  DEFAULT_BLINK_BRIGHTNESS = KEYPAD_LED_RES_MAX_BRIGHTNESS
  DEFAULT_BLINK_ON = 100  ' times in ms
  DEFAULT_BLINK_OFF = 200
  DEFAULT_BLINK_PAUSE = 1_000

  DEFAULT_PULSE_BRIGHTNESS = KEYPAD_LED_RES_MAX_BRIGHTNESS
  DEFAULT_PULSE_ON = 100 '3000
  DEFAULT_PULSE_OFF = 100 '3000
  
  DEFAULT_UNNEEDED_INTERVAL = 10000
                               
VAR

  long stack[32]
  long shortPause, longPause, brightnessAddress
  long phase, previousPhase, previousMode, timer
  long resolutionStep, pseudoPulseIncrement, activeBrightness, pseudoPulseLevel
  long intervalStartTime, activeInterval, blinkCounter
  long ms, unneededInterval
  byte ledPin, padPin52, padPin53, padPin51
  byte pwmFlag

{DAT

blinkBrightness         long DEFAULT_BLINK_BRIGHTNESS
pulseBrightness         long DEFAULT_PULSE_BRIGHTNESS
mode                    long PULSE_LED
blinkNumber             long DEFAULT_BLINK_NUMBER
pulseOn                 long DEFAULT_PULSE_ON
pulseOff                long DEFAULT_PULSE_OFF
blinkOn                 long DEFAULT_BLINK_ON
blinkOff                long DEFAULT_BLINK_OFF
blinkPause              long DEFAULT_BLINK_PAUSE
  
  
PUB Demo

  Start(LED_PIN, PAD_PIN_52, PAD_PIN_53, PAD_PIN_51, @blinkBrightness)

  repeat
    result := CheckButtons
    case mode
      PULSE_LED:
        case result
          UP_BUTTON:
            if pulseBrightness < KEYPAD_LED_RES_MAX_BRIGHTNESS
              pulseBrightness++
          DOWN_BUTTON:
            if pulseBrightness > 0
              pulseBrightness--
}              
PUB Start(ledPin_, padPin52_, padPin53_, padPin51_, brightnessAddress_)

  ledPin := ledPin_
  padPin52 := padPin52_
  padPin53 := padPin53_
  padPin51 := padPin51_
  brightnessAddress := brightnessAddress_ '+ (4 * BRIGHTNESS_BLINK_OFFSET)
  ms := clkfreq / 1000 
  unneededInterval := DEFAULT_UNNEEDED_INTERVAL * ms
  
  outa[padPin52] := 0
  outa[padPin53] := 0
  outa[padPin51] := 0
  dira[padPin52] := 0
  dira[padPin53] := 0
  dira[padPin51] := 0
  
  result := cognew(LightLed, @stack)
  waitcnt(clkfreq / 100 + cnt) 

PUB CheckButtons 
 
  if ina[padPin52] == 0
    return UP_BUTTON
      
  if ina[padPin53] == 0
    return DOWN_BUTTON
    
  if ina[padPin51] == 0
    return CENTER_BUTTON
    
  dira[padPin52] := 1
  if ina[padPin53] == 0
    dira[padPin52] := 0
    return RIGHT_BUTTON
  dira[padPin52] := 0
    
  dira[padPin51] := 1
  if ina[padPin53] == 0
    dira[padPin51] := 0
    return LEFT_BUTTON   
  dira[padPin51] := 0

  return NO_BUTTON
{
  ' offsets from brightnessAddress (all longs)
  #0, BRIGHTNESS_BLINK_OFFSET, BRIGHTNESS_PULSE_OFFSET, MODE_OFFSET, {RATE_OFFSET,} {
    } BLINK_NUMBER_OFFSET, LED_TIMER_PULSE_ON_OFFSET, LED_TIMER_PULSE_OFF_OFFSET, {
    } LED_TIMER_BLINK_ON_OFFSET, LED_TIMER_BLINK_OFF_OFFSET, {
    } LED_TIMER_BLINK_PAUSE_OFFSET
 
  ' phase enumeration
  #0, INCREASE_PHASE, DECREASE_PHASE, PAUSE_PHASE
  previousPhase, previousMode, timer  
}
PRI LightLed | localIndex, localMode
' Sometime move this to PASM

  dira[ledPin] := 1
  previousMode := -1
  resolutionStep := KEYPAD_LED_RES_MAX_BRIGHTNESS - 1
  intervalStartTime := cnt
  repeat
    ' Possible delay

    ' check for mode change
    if long[brightnessAddress][MODE_OFFSET] <> previousMode
      intervalStartTime := cnt
      phase := INCREASE_PHASE
      case long[brightnessAddress][MODE_OFFSET]
        OFF_LED:
          'outa[ledPin] := 0
          pwmFlag := OFF_PWM
          activeBrightness := 0
          activeInterval := unneededInterval
        PULSE_LED:
          'outa[ledPin] := 1
          pwmFlag := ON_PWM
          activeInterval := long[brightnessAddress][LED_TIMER_PULSE_ON_OFFSET]
          pseudoPulseIncrement := (long[brightnessAddress][BRIGHTNESS_PULSE_OFFSET] * {
          } PSEUDO_MULTIPLIER) / activeInterval
          pseudoPulseLevel := 0
          activeBrightness := 0
        BLINK_LED:
          'outa[ledPin] := 1
          pwmFlag := ON_PWM
          blinkCounter := 0
          activeInterval := long[brightnessAddress][LED_TIMER_BLINK_ON_OFFSET]
          activeBrightness := long[brightnessAddress][BRIGHTNESS_BLINK_OFFSET]
        SOLID_LED:
          'outa[ledPin] := 1
          pwmFlag := ON_PWM
          activeBrightness := long[brightnessAddress][BRIGHTNESS_BLINK_OFFSET]
          activeInterval := unneededInterval
          ' using same brightness for solid as blink for now
          
      previousMode := long[brightnessAddress][MODE_OFFSET]

    
    ' check for end of PWM cycle  
    if ++resolutionStep > KEYPAD_LED_RES_MAX_BRIGHTNESS
      resolutionStep := 0

    ' check to see if an interval is over
    if cnt - intervalStartTime > activeInterval
      'intervalStartTime := cnt
      intervalStartTime += activeInterval
      case long[brightnessAddress][MODE_OFFSET]
        'OFF_LED: ' nothing needs to change
        '  outa[ledPin] := 0
          'activeInterval := activeInterval
        PULSE_LED:
          case phase
            INCREASE_PHASE:
              phase := DECREASE_PHASE
              activeInterval := long[brightnessAddress][LED_TIMER_PULSE_ON_OFFSET]
              pseudoPulseIncrement := (long[brightnessAddress][BRIGHTNESS_PULSE_OFFSET] * {
              } -PSEUDO_MULTIPLIER) / activeInterval
            DECREASE_PHASE:
              phase := INCREASE_PHASE
              activeInterval := long[brightnessAddress][LED_TIMER_PULSE_OFF_OFFSET]
              pseudoPulseIncrement := (long[brightnessAddress][BRIGHTNESS_PULSE_OFFSET] * {
              } PSEUDO_MULTIPLIER) / activeInterval
        BLINK_LED:
          case phase
            INCREASE_PHASE:
              'outa[ledPin] := 0
              pwmFlag := OFF_PWM
              phase := DECREASE_PHASE
              activeInterval := long[brightnessAddress][LED_TIMER_BLINK_OFF_OFFSET] 
            DECREASE_PHASE:
              if ++blinkCounter > long[brightnessAddress][BLINK_NUMBER_OFFSET]
                blinkCounter := 0
                phase := PAUSE_PHASE
                activeInterval := long[brightnessAddress][LED_TIMER_BLINK_PAUSE_OFFSET]
                ' leave LED off
              else
                phase := INCREASE_PHASE
                activeInterval := long[brightnessAddress][LED_TIMER_BLINK_ON_OFFSET]
                'outa[ledPin] := 1
            PAUSE_PHASE:
              phase := INCREASE_PHASE
              activeInterval := long[brightnessAddress][LED_TIMER_BLINK_ON_OFFSET]
              'outa[ledPin] := 1
              pwmFlag := ON_PWM
        'SOLID_LED: ' nothing needs to change

      ' Does the brightness level need to be changed?
      if long[brightnessAddress][MODE_OFFSET] == PULSE_LED
        pseudoPulseLevel += pseudoPulseIncrement
        ' "PSEUDO_MULTIPLIER" used to reduce rounding errors
        activeBrightness := pseudoPulseLevel / PSEUDO_MULTIPLIER
       
    ' take care of PWM here
    if pwmFlag == ON_PWM
      if activeBrightness > resolutionStep 
        outa[ledPin] := 1
      else
        outa[ledPin] := 0  
    else
      outa[ledPin] := 0